program ex1b

  use pgmio

  implicit none

  include 'shmem.fh'

  integer, parameter :: nx = 360
  integer, parameter :: ny = 240

  integer, parameter :: numpes = 8

  integer, parameter :: nxlocal = nx
  integer, parameter :: nylocal = ny/numpes

  integer :: pe, mype

  double precision, save :: bigpic(nx, ny)
  double precision, save :: smallpic(nxlocal, nylocal)

  call shmem_init()

  mype = shmem_my_pe()

  if (numpes /= shmem_n_pes()) then

    if (mype == 0) then
      write(*,*) 'Error: compiled for ', numpes, & 
                 ' PE(s), but running on ', shmem_n_pes()
    end if

    call shmem_finalize()

    stop

  end if

  ! Even in Fortran, SHMEM PEs index from 0

  if (mype == 0)  write(*,*) 'Running on ', numpes, ' PE(s)'

  write(*,*) 'Hello from PE ', mype, ' out of ', numpes

  bigpic(:,:) = -1.0

  smallpic(:,:) = mype

! Need to synchronise to ensure smallpic is initialised on all PEs
! before the master PE reads them.

  call shmem_barrier_all()

  if (mype == 0) then  

    do pe = 0, numpes-1
      call shmem_double_get(bigpic(1, pe*nylocal + 1), smallpic(1,1), nxlocal*nylocal, pe)
    end do

    call pgmwrite('output.pgm', bigpic)
  
  end if

  call shmem_finalize()

end program ex1b
