program ex1c

  use pgmio

  implicit none

  include 'shmem.fh'

  integer, parameter :: nx = 360
  integer, parameter :: ny = 240

  integer, parameter :: numpes = 8

  integer, parameter :: nxlocal = nx
  integer, parameter :: nylocal = ny/numpes

  integer :: mype

  double precision, save :: bigpic(nx, ny)
  double precision, save :: smallpic(nxlocal, nylocal)

  call shmem_init()

  mype = shmem_my_pe()

  if (numpes /= shmem_n_pes()) then

    if (mype == 0) then
      write(*,*) 'Error: compiled for ', numpes, &
                 ' PE(s), but running on ', shmem_n_pes()
    end if

    call shmem_finalize()

    stop

  end if

  ! Even in Fortran, SHMEM PEs index from 0

  if (mype == 0)  write(*,*) 'Running on ', numpes, ' PE(s)'

  write(*,*) 'Hello from PE ', mype, ' out of ', numpes

  bigpic(:,:) = -1.0

  smallpic(:,:) = mype

! Need to synchronise to ensure bigpic is initialised on the master
! before any of the other PEs write to it.

  call shmem_barrier_all()  

  call shmem_double_put(bigpic(1,mype*nylocal+1), smallpic(1,1), nxlocal*nylocal, 0)

! Need to synchronise to ensure all PEs have finished writing
! to bigpic on the master before the master saves it.

  call shmem_barrier_all()

  if (mype == 0) call pgmwrite('output.pgm', bigpic)

  call shmem_finalize()
 
end program ex1c
