program ex2a

  use pgmio

  implicit none

  include 'shmem.fh'
  
  integer, parameter :: nx = 360
  integer, parameter :: ny = 240

  integer, parameter :: numpes = 8

  integer, parameter :: nxlocal = nx
  integer, parameter :: nylocal = ny/numpes

  integer :: mype

  double precision, save :: bigpic(nx, ny)

  double precision, save :: oldpic(0:nxlocal+1, 0:nylocal+1)
  double precision, save :: newpic(0:nxlocal+1, 0:nylocal+1)

  call shmem_init()

  mype = shmem_my_pe()

  if (numpes /= shmem_n_pes()) then

    if (mype == 0) then
      write(*,*) 'Error: compiled for ', numpes, &
                 ' PE(s), but running on ', shmem_n_pes()
    end if

    call shmem_finalize()

    stop

  end if

  if (mype == 0)  write(*,*) 'Running on ', numpes, ' PE(s)'

  write(*,*) 'Hello from PE ', mype, ' out of ', numpes

  oldpic(:,:) = 0.0
  newpic(:,:) = 0.0

! Read the picture on the master and copy to oldpic

  if (mype == 0) then

    call pgmread('picture360x240.pgm', bigpic)

! The following is only correct on a single PE - you need to change this

    oldpic(1:nx, 1:nylocal) = bigpic(1:nx, 1:nylocal)

  end if

! Overwrite bigpic to ensure that we have to do the output correctly

  bigpic(:,:) = -1.0

! This is where we should do the actual calculation of newpic as the
! second derivative (ie the edges) of oldpic. In this template, simply
! set newpic = oldpic.

  newpic(1:nx, 1:nylocal) = oldpic(1:nx, 1:nylocal)

! Now write the output on the master image.

  if (mype == 0) then  

! Copy newpic back to bigpic

    bigpic(1:nx, 1:nylocal) = newpic(1:nx, 1:nylocal)

    call pgmwrite('output.pgm', bigpic)

  end if

end program ex2a
